/*
  $Id: PooledSearchEntryResolver.java 2656 2013-03-11 22:41:10Z dfisher $

  Copyright (C) 2003-2013 Virginia Tech.
  All rights reserved.

  SEE LICENSE FOR MORE INFORMATION

  Author:  Middleware Services
  Email:   middleware@vt.edu
  Version: $Revision: 2656 $
  Updated: $Date: 2013-03-11 18:41:10 -0400 (Mon, 11 Mar 2013) $
*/
package org.ldaptive.auth;

import java.util.Arrays;
import org.ldaptive.Connection;
import org.ldaptive.LdapException;
import org.ldaptive.SearchOperation;
import org.ldaptive.SearchResult;
import org.ldaptive.pool.PooledConnectionFactory;
import org.ldaptive.pool.PooledConnectionFactoryManager;

/**
 * Looks up the LDAP entry associated with a user using a pool of LDAP
 * connections. Resolution will not occur using the connection that the user
 * attempted to bind on.
 *
 * @author  Middleware Services
 * @version  $Revision: 2656 $ $Date: 2013-03-11 18:41:10 -0400 (Mon, 11 Mar 2013) $
 */
public class PooledSearchEntryResolver extends AbstractSearchEntryResolver
  implements PooledConnectionFactoryManager
{

  /** Connection factory. */
  private PooledConnectionFactory factory;


  /** Default constructor. */
  public PooledSearchEntryResolver() {}


  /**
   * Creates a new pooled search entry resolver.
   *
   * @param  cf  connection factory
   */
  public PooledSearchEntryResolver(final PooledConnectionFactory cf)
  {
    setConnectionFactory(cf);
  }


  /**
   * Creates a new pooled search entry resolver.
   *
   * @param  cf  connection factory
   * @param  attrs  to return
   */
  public PooledSearchEntryResolver(
    final PooledConnectionFactory cf,
    final String... attrs)
  {
    setConnectionFactory(cf);
    setReturnAttributes(attrs);
  }


  /** {@inheritDoc} */
  @Override
  public PooledConnectionFactory getConnectionFactory()
  {
    return factory;
  }


  /** {@inheritDoc} */
  @Override
  public void setConnectionFactory(final PooledConnectionFactory cf)
  {
    factory = cf;
  }


  /** {@inheritDoc} */
  @Override
  protected SearchResult performLdapSearch(
    final Connection conn,
    final AuthenticationCriteria ac)
    throws LdapException
  {
    Connection pooledConn = null;
    try {
      pooledConn = factory.getConnection();

      final SearchOperation op = createSearchOperation(pooledConn);
      return
        op.execute(createSearchRequest(ac, getReturnAttributes())).getResult();
    } finally {
      if (pooledConn != null) {
        pooledConn.close();
      }
    }
  }


  /** {@inheritDoc} */
  @Override
  public String toString()
  {
    return
      String.format(
        "[%s@%d::factory=%s, returnAttributes=%s]",
        getClass().getName(),
        hashCode(),
        factory,
        Arrays.toString(getReturnAttributes()));
  }
}
